# -*- coding: utf-8 -*-
from facturacion_electronica.libro_boletas import Boletas
from facturacion_electronica.cesion import Cesion
from facturacion_electronica.conexion import Conexion
from facturacion_electronica.consumo_folios import ConsumoFolios
from facturacion_electronica.documento import Documento
from facturacion_electronica.emisor import Emisor
from facturacion_electronica.envio import Envio
from facturacion_electronica.firma import Firma
from facturacion_electronica.libro import Libro
import json
import csv
import base64
import logging
_logger = logging.getLogger(__name__)


envios = {}


def timbrar_json(json_str=None):
    try:
        data = json.loads(json_str)
        return timbrar(data)
    except:
        return


def procesar_documento(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    verify = vals.get('verify', True)
    test = vals.get('test', False)
    _documentos = []
    for docs in vals.get('Documento'):
        TipoDTE = docs.get('TipoDTE', 33)
        caf_file = docs.get('caf_file', [])
        for docData in docs["documentos"]:
            docu = Documento(
                            docData,
                            emisor=emisor,
                            resumen=False,
                            tipo_dte=TipoDTE,
                        )
            docu._firma = firma
            docu.verify = verify
            docu.test = test
            docu.caf_file = caf_file
            _documentos.append(docu)
    return _documentos


def timbrar(vals):
    _dtes = False
    if vals.get('Documento'):
        _dtes = procesar_documento(vals)
    respuesta = []
    if _dtes:
        dtes = sorted(_dtes, key=lambda t: int(t.NroDTE))
        errores = []
        for dte in dtes:
            resp = {
                'NroDTE': dte.NroDTE,
                'TipoDTE': dte.TipoDTE,
                'Folio': dte.Folio,
                'FechaEmis': dte.FechaEmis,
                'MntExe': dte.MntExe,
                'MntNeto': dte.MntNeto,
                'MntIVA': dte.MntIVA,
                'MntTotal': dte.MntTotal,
                'ImptoReten': dte.ImptoReten
            }
            try:
                if dte.sii_xml_request:
                    dte.timbrar_xml()
                else:
                    dte.timbrar()
                resp.update({
                    'sii_xml_dte': dte.sii_xml_request,
                    'sii_barcode_img': dte.sii_barcode_img,
                    'sii_barcode': dte.sii_barcode.decode('ISO-8859-1'),
                })
            except Exception as e:
                resp.update({
                    'error': str(e),
                })
            respuesta.append(resp)
    return respuesta


def xml_envio(vals):
    envio = Envio(vals)
    envio.test = True
    respuesta = envio.do_dte_send()
    return respuesta


def key_check(vals):
    return (vals['Emisor']['RUTEmisor'],
            vals['Emisor']['Modo'],
            vals['firma_electronica']['rut_firmante'],
            vals.get('ID', 'SetDoc'))

def check_en_envio(key):
    if envios.get(key):
        return True
    envios[key] = True
    return False


def timbrar_y_enviar(vals):
    key = key_check(vals)
    if check_en_envio(key):
        return 'DTE Ya se encuentra en envío'
    respuesta = {}
    try:
        envio = Envio(vals)
        respuesta = envio.do_dte_send()
    except Exception as e:
        _logger.warning("Error en timbrar_y_enviar", exc_info=True)
        respuesta["errores"] = [str(e)]
    del envios[key]
    return respuesta


def timbrar_y_enviar_cesion(vals):
    key = key_check(vals)
    if check_en_envio(key):
        return 'Cesión Ya se encuentra en envío'
    respuesta = ''
    try:
        envio = Envio(vals)
        respuesta = envio.do_envio_cesion()
    except:
        _logger.warning("Error en timbrar_y_enviar_cesion", exc_info=True)
    del envios[key]
    return respuesta

def enviar_xml(vals):
    key = key_check(vals)
    if check_en_envio(key):
        return 'DTE Ya se encuentra en envío'
    respuesta = ''
    try:
        envio = Envio(vals)
        respuesta = envio.enviar_xml()
    except Exception as e:
        _logger.warning("Error en envíar_xml", exc_info=True)
    del envios[key]
    return respuesta


def cf_json(json_str=None):
    try:
        data = json.loads(json_str)
        return consumo_folios(data)
    except:
        return


def consumo_folios(vals):
    key = key_check(vals)
    if check_en_envio(key):
        return 'CF Ya se encuentra en envío'
    resultado = {}
    try:
        envio = Envio(vals)
        resultado = envio.do_consumo_folios_send()
    except Exception as e:
        msg = "Error en envío CF: %s" % str(e)
        _logger.warning(msg, exc_info=True)
        resultado['errors'] = msg
    del envios[key]
    return resultado


def consumo_folios_resumen(data, folios=False):
    test = data['test'] if 'test' in data else False
    emisor = Emisor(data["Emisor"])
    firma = Firma(data["firma_electronica"])
    fechas = {}
    if folios:
        i = 0
        for row in folios:
            if i > 0:
                if row[0] not in fechas:
                    fechas[row[0]] = ConsumoFolios(emisor, firma)
                if 'Correlativo' in data:
                    fechas[row[0]].set_correlativo(data['Correlativo'])
                if 'SecEnvio' in data:
                    fechas[row[0]].set_sec_envio(data['SecEnvio'])
                docu = Documento({
                    'TasaImp': data["Emisor"]['ValorIva'],
                    'Fecha': row[0],
                    'Folio': int(row[2]),
                    'MntNeto': int(row[3]),
                    'MntIVA': int(row[4]),
                    'MntExento': int(row[5]),
                    'MntTotal': int(row[6]),
                    'Anulado': row[7],
                    }, int(row[1]), resumen=True)
                fechas[row[0]].set_docu(docu)
            else:
                cabezera = row
            i += 1
    elif 'Documentos' in data:
        for r in data['Documentos']:
            for d in r['documentos']:
                fecha = d['Encabezado']['IdDoc']['Fecha']
                if fecha not in fechas:
                    fechas[fecha] = ConsumoFolios(emisor, firma)
                if 'Correlativo' in data:
                    fechas[fecha].set_correlativo(data['Correlativo'])
                if 'SecEnvio' in data:
                    fechas[fecha].set_sec_envio(data['SecEnvio'])
                docu = Documento(d, r['TipoDTE'])
                fechas[fecha].set_docu(docu)
    else:
        cf = ConsumoFolios(emisor, firma)
        cf.FchInicio = data['FechaInicio']
        cf.FchFinal = data['FechaFinal']
        if 'Correlativo' in data:
            cf.set_correlativo(data['Correlativo'])
        if 'SecEnvio' in data:
            cf.set_sec_envio(data['SecEnvio'])
        envio = Envio(emisor, firma, test=test)
        envio.setLibro(cf)
        res = envio.do_consumo_folios_send()
        res.update({
                'correlativo': cf.Correlativo,
                'fecha_inicio': cf.FchInicio,
                'fecha_final': cf.FchFinal,
                'total_neto': cf.total_neto,
                'total_exento': cf.total_exento,
                'total_iva': cf.total_iva,
                'total': cf.total,
                'total_boletas': cf.total_boletas,

        })
        return [res]
    cfs = []
    respuestas = []
    envio = Envio(emisor, firma, test=test)
    for key, cf in fechas.items():
        cf.validar()
        cfs.append(cf.sii_xml_request)
        envio.setLibro(cf)
        res = envio.do_consumo_folios_send()
        res.update({
                'correlativo': cf.Correlativo,
                'fecha_inicio': cf.FchInicio,
                'fecha_final': cf.FchFinal,
                'total_neto': cf.total_neto,
                'total_exento': cf.total_exento,
                'total_iva': cf.total_iva,
                'total': cf.total,
                'total_boletas': cf.total_boletas,
                'rangos': cf.get_rangos(),

        })
        respuestas.append(res)
    return respuestas


def libro(vals=None):
    key = key_check(vals)
    if check_en_envio(key):
        return "Libro ya en envío"
    resultado = {}
    try:
        envio = Envio(vals)
        resultado = envio.do_libro_send()
    except Exception as e:
        _logger.warning("Error en envío libro", exc_info=True)
        resultado['errores'] = str(e)
    del envios[key]
    return resultado


def libro_resumen(dicDocs, csv=False):
    firma = Firma(dicDocs["firma_electronica"])
    emisor = Emisor(dicDocs["Emisor"])
    libro = Libro(emisor, firma)
    libro.set_periodo_tributario(dicDocs["periodo_tributario"])
    libro.set_tipo_operacion(dicDocs["tipo_operacion"])
    libro.set_tipo_libro(dicDocs["tipo_libro"])
    test = dicDocs['test'] if 'test' in dicDocs else False
    envio = Envio(emisor, firma, test=test)
    respuesta = []
    if csv:
        libro = _libro_csv(libro, csv)
    else:
        for dteDoc in dicDocs["Documentos"]:
            for docData in dteDoc["documentos"]:
                docu = Documento(docData, dteDoc["TipoDTE"])
                libro.set_docu(docu)
    libro.validar()
    envio.setLibro(libro)
    respuesta = envio.do_libro_send()
    return respuesta


def _libro_csv(libro, csv=None):
    '''
    0 = Tipo Doc;                           11 = Cod IVA no Rec;                22 = Tabacos - Cigarrillos;
    1 = Folio;                              12 = Monto IVA no Rec;              23 = Tabacos - Elaborados;
    2 = Rut Contraparte;                    13 = IVA Uso Común;                 24 = Impuesto a Vehiculos Automóviles;
    3 = Tasa Impuesto;                      14 = Cod Otro Imp (Con Crédito);    25 = Codigo sucursal SII;
    4 = Razón Social Contraparte;           15 = Tasa Otro Imp (Con Crédito);   26 = Numero Interno;Emisor/Receptor;
    5 = Tipo Impuesto[1=IVA:2=LEY 18211];   16 = Monto Otro Imp (Con Crédito);  27 = Emisor Receptor;
    6 = Fecha Emisión;                      17 = Monto Otro Imp Sin Crédito;    28 = Monto Total;
    7 = Anulado[A];                         18 = Monto Activo Fijo;
    8 = Monto Exento;                       19 = Monto IVA Activo Fijo;
    9 = Monto Neto;                         20 = IVA No Retenido;
    10 = Monto IVA (Recuperable);           21 = Tabacos - Puros;
    '''
    i = 0
    for row in csv:
        if not row:
            continue
        if i > 0:
            if int(row[0]) not in [39, 41, 35]:
                docu = Documento({
                    'Folio': int(row[1]),
                    'Encabezado': {
                            'Receptor': {
                                'RUTRecep': row[2],
                                'RznSocRecep': row[4]
                            }
                    },
                    'TasaImp': int(row[3]),
                    'TpoImp': int(row[5]),
                    'Fecha': row[6],
                    'Anulado': row[7],
                    'MntExento': int(row[8]) if row[8] else False,
                    'MntNeto': int(row[9]) if row[9] else False,
                    'MntIVA': int(row[10]) if row[10] else False,
                    'CodIVANoRec': int(row[11]) if row[11] else False,
                    'IVANoRec':  int(row[12]) if row[12] else False,
                    'IVAUsoComun': int(row[13]) if row[13] else False,
                    'CodOtroImp': int(row[14]) if row[14] else False,
                    'TasaOtroImp': int(row[15]) if row[15] else False,
                    'MontoOtroImp': int(row[16]) if row[16] else False,
                    'MntTotal': int(row[17]) if row[17] else False,
                    'MontoOtroImpNoRec': int(row[18]) if row[18] else False,
                    'MontoActivoFijo': int(row[19]) if row[19] else False,
                    'MontoIVAActivoFijo': int(row[20]) if row[20] else False,
                    'IVANoRetenido': int(row[21]) if row[21] else False,
                    'TabacosPuros': int(row[22]) if row[22] else False,
                    'TabacosCigarrillos': int(row[23]) if row[23] else False,
                    'TabacosElaborados': int(row[24]) if row[24] else False,
                    'Vehiculos': int(row[25]) if row[25] else False,
                    'CodSucur': int(row[26]) if row[26] else False,
                    'NumeroInterno': int(row[27]) if row[27] else False,
                    'EmisorReceptor': int(row[28]) if row[28] else False,

                    }, int(row[0]), resumen=True)
            else:
                docu = Boletas({
                        'Folio': int(row[1]),
                        'TasaImp': int(row[3]),
                        'MntExento': int(row[8]) if row[8] else False,
                        'MntNeto': int(row[9]) if row[9] else False,
                        'MntIVA': int(row[10]) if row[10] else False,
                        'MntTotal': int(row[17]) if row[17] else False,
                    }, int(row[0]))
            libro.set_docu(docu)
        else:
            cabezera = row
        i += 1
    return libro


def consulta_estado_envio(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    conex = Conexion(emisor, firma, vals.get('api', False))
    if vals.get('cesion'):
        conex.cesion = vals['cesion']
    return conex.consulta_estado_envio(vals["codigo_envio"])

def reenvio_correo_envio(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    conex = Conexion(emisor, firma, vals.get('api', False))
    return conex.reenvio_correo_envio(vals["codigo_envio"])


def consulta_estado_dte(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    respuesta = {}
    for d in vals['Documento']:
        for r in d['documentos']:
            tipoDoc = int(d["TipoDTE"])
            cesion = r.get('cesion', False)
            if cesion:
                del r['cesion']
                documento = Cesion(r)
                documento._dte_emisor = emisor
            else:
                documento = Documento(
                    r,
                    emisor=emisor,
                    resumen=True)
            documento.TipoDTE = tipoDoc
            conexion = Conexion(emisor, firma, vals.get('api', documento.es_boleta()))
            conexion.cesion = cesion
            respuesta[documento.ID] = conexion.consulta_estado_dte(
                documento)
    return respuesta

def consulta_estado_cesion_relac(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    respuesta = {}
    for d in vals['Documento']:
        for r in d['documentos']:
            tipoDoc = int(d["TipoDTE"])
            cesion = r.get('cesion', False)
            del r['cesion']
            documento = Cesion(r)
            documento._dte_emisor = emisor
            documento.TipoDTE = tipoDoc
            conexion = Conexion(emisor, firma)
            conexion.cesion = cesion
            respuesta[documento.ID] = conexion.consulta_estado_cesion_relac(
                documento)
    return respuesta

def ingreso_reclamo_documento(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    respuesta = {}
    for d in vals['DTEClaim']:
        key = "RUT%sT%sF%s" %(d['RUTEmisor'], d['TipoDTE'], d['Folio'])
        conexion = Conexion(emisor, firma)
        respuesta[key] = conexion.set_dte_claim(d)
    return respuesta


def consulta_reclamo_documento(vals):
    firma = Firma(vals["firma_electronica"])
    emisor = Emisor(vals["Emisor"])
    respuesta = {}
    for d in vals['DTEClaim']:
        key = "RUT%sT%sF%s" %(d['RUTEmisor'], d['TipoDTE'], d['Folio'])
        conexion = Conexion(emisor, firma)
        respuesta[key] = conexion.get_dte_claim(d)
    return respuesta

def recepcion_xml(vals):
    vals.update({'test': True})
    try:
        resp = Envio(vals)
        respuestas = resp.do_receipt_deliver()
    except Exception as e:
        msg = "No se pudo crear recepcion XML: " + str(e)
        _logger.warning(msg, exc_info=True)
        respuestas = {
            "error": msg
        }
    return respuestas


def recepcion_mercaderias(vals):
    vals.update({'test': True})
    try:
        resp = Envio(vals)
        respuesta = resp.do_recep_merc()
    except Exception as e:
        msg = "No se pudo crear recepción mercadería: " + str(e)
        _logger.warning(msg, exc_info=True)
        respuesta = {
            "error": msg
        }
    return respuesta


def validacion_comercial(vals):
    vals.update({'test': True})
    try:
        resp = Envio(vals)
        respuesta = resp.do_validar_com()
    except Exception as e:
        msg = "No se pudo crear validación Comercial: " + str(e)
        _logger.warning(msg, exc_info=True)
        respuesta = {
            "error": msg
        }
    return respuesta


def test(vals):
    if vals is None:
        return "r null"
    else:
        return "your firma is : " + vals["firma"]


def main():
    test({})


def ejemplo_libro():
    docEjemplo = json.load(open("ejemplos/ejemplo_libro_ventas.json"))
    libro(docEjemplo)


def ejemplo_cf():
    docEjemplo = json.load(open("ejemplos/ejemplo_consumo_folios.json"))
    consumo_folios(docEjemplo)


def ejemplo_cf_csv():
    data = json.load(open("ejemplos/ejemplo_consumo_folios_csv.json"))
    folios = csv.reader(base64.b64decode(data['consumo_folios']).splitlines(), csv.excel, delimiter=',', quotechar='|')
    i = 0
    listado = ['Fecha', 'Folio', 'RUTRecep', 'MntNeto', 'MntIVA', 'MntExento', 'MntTotal']
    firma = Firma(data["firma_electronica"])
    emisor = Emisor(data["Emisor"])
    fechas = {}
    for row in folios:
        if i >0:
            if row[0] not in fechas:
                fechas[row[0]] = ConsumoFolios(emisor, firma)
            else:
                docu = Documento({
                    'Fecha': row[0],
                    'Folio': int(row[2]),
                    'MntNeto': float(row[3]),
                    'MntIVA': float(row[4]),
                    'MntExento': float(row[5]),
                    'MntTotal': float(row[6]),
                    'Anulado': row[7],
                    }, int(row[1]), resumen=True)
                fechas[row[0]].set_docu(docu)

        else:
            cabezera = row
        i += 1
    for key, cf in fechas.items():
        cf.validar()
        print (cf.sii_xml_request)


if __name__ == "__main__":
    main()
